@@ -1,21 +1,21 @@
 import collections
 import functools
 import itertools
+import math
 import operator
 import types
-import warnings
+from typing import Callable, Iterable, Mapping, Union
 import django
 from django.db import IntegrityError, models, transaction
 from django.db.models import functions
-from django.utils import six
-map = six.moves.map
+
 try:  # pragma: no cover
     import django.contrib.gis.db.models.functions
     import django.contrib.gis.db.models as gis
 except Exception:  # pragma: no cover
     gis = None
 
-__version__ = '0.6'
+__version__ = '1.2'
 
 
 def update_wrapper(wrapper, name):
@@ -23,45 +23,48 @@ def update_wrapper(wrapper, name):
     return wrapper
 
 
-def method(lookup):
+def eq(lookup):
     return update_wrapper(lambda self, value: self.__eq__(value, '__' + lookup), lookup)
 
 
-def starmethod(lookup):
-    return update_wrapper(lambda self, *values: self.__eq__(values, '__' + lookup), lookup)
-
-
-class Lookup(object):
+class Lookup:
     """Mixin for field lookups."""
-    __ne__ = method('ne')
-    __lt__ = method('lt')
-    __le__ = method('lte')
-    __gt__ = method('gt')
-    __ge__ = method('gte')
-    iexact = method('iexact')
-    icontains = method('icontains')
-    startswith = method('startswith')
-    istartswith = method('istartswith')
-    endswith = method('endswith')
-    iendswith = method('iendswith')
-    regex = method('regex')
-    iregex = method('iregex')
-    in_ = starmethod('in')
-    range = starmethod('range')
+
+    __ne__ = eq('ne')
+    __lt__ = eq('lt')
+    __le__ = eq('lte')
+    __gt__ = eq('gt')
+    __ge__ = eq('gte')
+    iexact = eq('iexact')
+    icontains = eq('icontains')
+    startswith = eq('startswith')
+    istartswith = eq('istartswith')
+    endswith = eq('endswith')
+    iendswith = eq('iendswith')
+    regex = eq('regex')
+    iregex = eq('iregex')
+    isin = eq('in')
     # spatial lookups
-    contained = method('contained')
-    coveredby = method('coveredby')
-    covers = method('covers')
-    crosses = method('crosses')
-    disjoint = method('disjoint')
-    equals = method('equals')  # __eq__ is taken
-    intersects = method('intersects')  # __and__ is ambiguous
-    relate = starmethod('relate')
-    touches = method('touches')
-    __lshift__ = left = method('left')
-    __rshift__ = right = method('right')
-    above = method('strictly_above')
-    below = method('strictly_below')
+    contained = eq('contained')
+    coveredby = eq('coveredby')
+    covers = eq('covers')
+    crosses = eq('crosses')
+    disjoint = eq('disjoint')
+    equals = eq('equals')  # __eq__ is taken
+    intersects = eq('intersects')  # __and__ is ambiguous
+    touches = eq('touches')
+    __lshift__ = left = eq('left')
+    __rshift__ = right = eq('right')
+    above = eq('strictly_above')
+    below = eq('strictly_below')
+
+    def range(self, *values):
+        """range"""
+        return self.__eq__(values, '__range')
+
+    def relate(self, *values):
+        """relate"""
+        return self.__eq__(values, '__relate')
 
     @property
     def is_valid(self):
@@ -74,7 +77,9 @@ class Lookup(object):
         :param properly: `contains_properly`
         :param bb: bounding box, `bbcontains`
         """
-        return self.__eq__(value, '__{}contains{}'.format('bb' * bool(bb), '_properly' * bool(properly)))
+        properly = '_properly' * bool(properly)
+        bb = 'bb' * bool(bb)
+        return self.__eq__(value, f'__{bb}contains{properly}')
 
     def overlaps(self, geom, position='', bb=False):
         """Return whether field `overlaps` with geometry .
@@ -82,7 +87,8 @@ class Lookup(object):
         :param position: `overlaps_{left, right, above, below}`
         :param bb: bounding box, `bboverlaps`
         """
-        return self.__eq__(geom, '__{}overlaps_{}'.format('bb' * bool(bb), position).rstrip('_'))
+        bb = 'bb' * bool(bb)
+        return self.__eq__(geom, f'__{bb}overlaps_{position}'.rstrip('_'))
 
     def within(self, geom, distance=None):
         """Return whether field is `within` geometry.
@@ -109,21 +115,21 @@ def transform(lookup, func, value):
 
 
 class MetaF(type):
-    def __getattr__(cls, name):
-        if name == 'name':
-            raise AttributeError("'name' is a reserved attribute")
+    def __getattr__(cls, name: str) -> 'F':
+        if name in ('name', '__slots__'):
+            raise AttributeError(f"'{name}' is a reserved attribute")
         return cls(name)
 
-    def any(cls, exprs):
+    def any(cls, exprs: Iterable[models.Q]) -> models.Q:
         """Return ``Q`` OR object."""
         return functools.reduce(operator.or_, exprs)
 
-    def all(cls, exprs):
+    def all(cls, exprs: Iterable[models.Q]) -> models.Q:
         """Return ``Q`` AND object."""
         return functools.reduce(operator.and_, exprs)
 
 
-class F(six.with_metaclass(MetaF, models.F, Lookup)):
+class F(models.F, Lookup, metaclass=MetaF):
     """Create ``F``, ``Q``, and ``Func`` objects with expressions.
 
     ``F`` creation supported as attributes:
@@ -137,7 +143,26 @@ class F(six.with_metaclass(MetaF, models.F, Lookup)):
     ``Func`` objects also supported as methods:
     ``F.user.created.min()`` == ``Min('user__created')``.
     """
-    lookups = dict(length=functions.Length, lower=functions.Lower, upper=functions.Upper)
+
+    lookups = dict(
+        length=functions.Length,
+        lower=functions.Lower,
+        upper=functions.Upper,
+        chr=functions.Chr,
+        ord=functions.Ord,
+        acos=functions.ACos,
+        asin=functions.ASin,
+        atan=functions.ATan,
+        atan2=functions.ATan2,
+        cos=functions.Cos,
+        cot=functions.Cot,
+        degrees=functions.Degrees,
+        exp=functions.Exp,
+        radians=functions.Radians,
+        sin=functions.Sin,
+        sqrt=functions.Sqrt,
+        tan=functions.Tan,
+    )
     coalesce = method(functions.Coalesce)
     concat = method(functions.Concat)  # __add__ is taken
     min = method(models.Min)
@@ -152,24 +177,37 @@ class F(six.with_metaclass(MetaF, models.F, Lookup)):
     cast = method(functions.Cast)
     extract = method(functions.Extract)
     trunc = method(functions.Trunc)
-    if django.VERSION >= (2,):
-        cume_dist = method(functions.CumeDist)
-        dense_rank = method(functions.DenseRank)
-        first_value = method(functions.FirstValue)
-        lag = method(functions.Lag)
-        last_value = method(functions.LastValue)
-        lead = method(functions.Lead)
-        nth_value = method(functions.NthValue)
-        ntile = staticmethod(functions.Ntile)
-        percent_rank = method(functions.PercentRank)
-        rank = method(functions.Rank)
-        row_number = method(functions.RowNumber)
-    if django.VERSION >= (2, 1):
-        lookups.update(chr=functions.Chr, ord=functions.Ord)
-        strip = method(functions.Trim)
-        lstrip = method(functions.LTrim)
-        rstrip = method(functions.RTrim)
-        repeat = method(functions.Repeat)
+    cume_dist = method(functions.CumeDist)
+    dense_rank = method(functions.DenseRank)
+    first_value = method(functions.FirstValue)
+    lag = method(functions.Lag)
+    last_value = method(functions.LastValue)
+    lead = method(functions.Lead)
+    nth_value = method(functions.NthValue)
+    ntile = staticmethod(functions.Ntile)
+    percent_rank = method(functions.PercentRank)
+    rank = method(functions.Rank)
+    row_number = method(functions.RowNumber)
+    strip = method(functions.Trim)
+    lstrip = method(functions.LTrim)
+    rstrip = method(functions.RTrim)
+    repeat = method(functions.Repeat)
+    nullif = method(functions.NullIf)
+    __reversed__ = method(functions.Reverse)
+    __abs__ = method(functions.Abs)
+    __ceil__ = method(functions.Ceil)
+    __floor__ = method(functions.Floor)
+    __mod__ = method(functions.Mod)
+    pi = functions.Pi()
+    __pow__ = method(functions.Power)
+    __round__ = method(functions.Round)
+    if django.VERSION >= (3,):
+        lookups.update(sign=functions.Sign, md5=functions.MD5)
+        sha1 = method(functions.SHA1)
+        sha224 = method(functions.SHA224)
+        sha256 = method(functions.SHA256)
+        sha384 = method(functions.SHA384)
+        sha512 = method(functions.SHA512)
     if gis:  # pragma: no cover
         area = property(gis.functions.Area)
         geojson = method(gis.functions.AsGeoJSON)
@@ -180,7 +218,6 @@ class F(six.with_metaclass(MetaF, models.F, Lookup)):
         centroid = property(gis.functions.Centroid)
         difference = method(gis.functions.Difference)
         envelope = property(gis.functions.Envelope)
-        force_rhr = method(gis.functions.ForceRHR)
         geohash = method(gis.functions.GeoHash)  # __hash__ requires an int
         intersection = method(gis.functions.Intersection)
         make_valid = method(gis.functions.MakeValid)
@@ -196,29 +233,46 @@ class F(six.with_metaclass(MetaF, models.F, Lookup)):
         transform = method(gis.functions.Transform)
         translate = method(gis.functions.Translate)
         union = method(gis.functions.Union)
+        azimuth = method(gis.functions.Azimuth)
+        line_locate_point = method(gis.functions.LineLocatePoint)
+        force_polygon_cw = method(gis.functions.ForcePolygonCW)
 
         @method
         class distance(gis.functions.Distance):
             """Return ``Distance`` with support for lookups: <, <=, >, >=, within."""
+
             __lt__ = method(transform, 'distance_lt')
             __le__ = method(transform, 'distance_lte')
             __gt__ = method(transform, 'distance_gt')
             __ge__ = method(transform, 'distance_gte')
             within = method(transform, 'dwithin')
 
-    def __getattr__(self, name):
+    def __getattr__(self, name: str) -> 'F':
         """Return new `F`_ object with chained attribute."""
         return type(self)('{}__{}'.format(self.name, name))
 
-    def __eq__(self, value, lookup=''):
+    def __eq__(self, value, lookup: str = '') -> models.Q:
         """Return ``Q`` object with lookup."""
+        if not lookup and type(value) is models.F:
+            return self.name == value.name
         return models.Q(**{self.name + lookup: value})
 
-    def __call__(self, *args, **extra):
+    def __ne__(self, value) -> models.Q:
+        """Allow __ne=None lookup without custom queryset."""
+        if value is None:
+            return self.__eq__(False, '__isnull')
+        return self.__eq__(value, '__ne')
+
+    __hash__ = models.F.__hash__
+
+    def __call__(self, *args, **extra) -> models.Func:
         name, _, func = self.name.rpartition('__')
         return self.lookups[func](name, *args, **extra)
 
-    def __getitem__(self, slc):
+    def __iter__(self):
+        raise TypeError("'F' object is not iterable")
+
+    def __getitem__(self, slc: slice) -> models.Func:
         """Return field ``Substr`` or ``Right``."""
         assert (slc.stop or 0) >= 0 and slc.step is None
         start = slc.start or 0
@@ -228,55 +282,65 @@ class F(six.with_metaclass(MetaF, models.F, Lookup)):
         size = slc.stop and max(slc.stop - start, 0)
         return functions.Substr(self, start + 1, size)
 
+    def __rmod__(self, value):
+        return functions.Mod(value, self)
+
+    def __rpow__(self, value):
+        return functions.Power(value, self)
+
     @method
     def count(self='*', **extra):
         """Return ``Count`` with optional field."""
         return models.Count(getattr(self, 'name', self), **extra)
 
-    def find(self, sub, **extra):
+    def find(self, sub, **extra) -> models.Expression:
         """Return ``StrIndex`` with ``str.find`` semantics."""
         return functions.StrIndex(self, Value(sub), **extra) - 1
 
-    def replace(self, old, new='', **extra):
+    def replace(self, old, new='', **extra) -> models.Func:
         """Return ``Replace`` with wrapped values."""
         return functions.Replace(self, Value(old), Value(new), **extra)
 
-    def ljust(self, width, fill=' ', **extra):
+    def ljust(self, width: int, fill=' ', **extra) -> models.Func:
         """Return ``LPad`` with wrapped values."""
         return functions.LPad(self, width, Value(fill), **extra)
 
-    def rjust(self, width, fill=' ', **extra):
+    def rjust(self, width: int, fill=' ', **extra) -> models.Func:
         """Return ``RPad`` with wrapped values."""
         return functions.RPad(self, width, Value(fill), **extra)
 
+    def log(self, base=math.e, **extra) -> models.Func:
+        """Return ``Log``, by default ``Ln``."""
+        return functions.Log(self, base, **extra)
 
-def method(func):
+
+def reduce(func):
     return update_wrapper(lambda self: self.reduce(func), func.__name__)
 
 
-def binary(func):
+def field(func):
     return update_wrapper(lambda self, value: func(models.F(*self._fields), value), func.__name__)
 
 
 class QuerySet(models.QuerySet, Lookup):
-    min = method(models.Min)
-    max = method(models.Max)
-    sum = method(models.Sum)
-    mean = method(models.Avg)
-    var = method(models.Variance)
-    std = method(models.StdDev)
-    __add__ = binary(operator.add)
-    __sub__ = binary(operator.sub)
-    __mul__ = binary(operator.mul)
-    __truediv__ = __div__ = binary(operator.truediv)
-    __mod__ = binary(operator.mod)
-    __pow__ = binary(operator.pow)
+    min = reduce(models.Min)
+    max = reduce(models.Max)
+    sum = reduce(models.Sum)
+    mean = reduce(models.Avg)
+    var = reduce(models.Variance)
+    std = reduce(models.StdDev)
+    __add__ = field(operator.add)
+    __sub__ = field(operator.sub)
+    __mul__ = field(operator.mul)
+    __truediv__ = field(operator.truediv)
+    __mod__ = field(operator.mod)
+    __pow__ = field(operator.pow)
     if gis:  # pragma: no cover
-        collect = method(gis.Collect)
-        extent = method(gis.Extent)
-        extent3d = method(gis.Extent3D)
-        make_line = method(gis.MakeLine)
-        union = method(gis.Union)
+        collect = reduce(gis.Collect)
+        extent = reduce(gis.Extent)
+        extent3d = reduce(gis.Extent3D)
+        make_line = reduce(gis.MakeLine)
+        union = reduce(gis.Union)
 
     @property
     def _flat(self):
@@ -284,7 +348,7 @@ class QuerySet(models.QuerySet, Lookup):
 
     @property
     def _named(self):
-        return issubclass(self._iterable_class, getattr(models.query, 'NamedValuesListIterable', ()))
+        return issubclass(self._iterable_class, models.query.NamedValuesListIterable)
 
     def __getitem__(self, key):
         """Allow column access by field names, expressions, or ``F`` objects.
@@ -296,22 +360,21 @@ class QuerySet(models.QuerySet, Lookup):
         ``qs[Q_obj]`` provisionally returns filtered `QuerySet`_
         """
         if isinstance(key, tuple):
-            kwargs = {'named': True} if django.VERSION >= (2,) else {}
-            return self.values_list(*map(extract, key), **kwargs)
+            return self.values_list(*map(extract, key), named=True)
         key = extract(key)
-        if isinstance(key, six.string_types + (models.Expression,)):
+        if isinstance(key, (str, models.Expression)):
             return self.values_list(key, flat=True)
         if isinstance(key, models.Q):
             return self.filter(key)
-        return super(QuerySet, self).__getitem__(key)
+        return super().__getitem__(key)
 
     def __setitem__(self, key, value):
         """Update a single column."""
         self.update(**{key: value})
 
-    def __eq__(self, value, lookup=''):
+    def __eq__(self, value, lookup: str = '') -> 'QuerySet':
         """Return `QuerySet`_ filtered by comparison to given value."""
-        field, = self._fields
+        (field,) = self._fields
         return self.filter(**{field + lookup: value})
 
     def __contains__(self, value):
@@ -323,7 +386,7 @@ class QuerySet(models.QuerySet, Lookup):
     def __iter__(self):
         """Iteration extended to support :meth:`groupby`."""
         if not hasattr(self, '_groupby'):
-            return super(QuerySet, self).__iter__()
+            return super().__iter__()
         size = len(self._groupby)
         rows = self[self._groupby + self._fields].order_by(*self._groupby).iterator()
         groups = itertools.groupby(rows, key=operator.itemgetter(*range(size)))
@@ -333,11 +396,11 @@ class QuerySet(models.QuerySet, Lookup):
             getter = lambda tup: Row(*tup[size:])  # noqa
         return ((key, map(getter, values)) for key, values in groups)
 
-    def items(self, *fields, **annotations):
+    def items(self, *fields, **annotations) -> 'QuerySet':
         """Return annotated ``values_list``."""
         return self.annotate(**annotations)[fields + tuple(annotations)]
 
-    def groupby(self, *fields, **annotations):
+    def groupby(self, *fields, **annotations) -> 'QuerySet':
         """Return a grouped `QuerySet`_.
 
         The queryset is iterable in the same manner as ``itertools.groupby``.
@@ -347,25 +410,23 @@ class QuerySet(models.QuerySet, Lookup):
         qs._groupby = fields + tuple(annotations)
         return qs
 
-    def annotate(self, *args, **kwargs):
+    def annotate(self, *args, **kwargs) -> 'QuerySet':
         """Annotate extended to also handle mapping values, as a `Case`_ expression.
 
         :param kwargs: ``field={Q_obj: value, ...}, ...``
 
         As a provisional feature, an optional ``default`` key may be specified.
         """
-        kwargs.update((field, Case.defaultdict(value)) for field, value in kwargs.items()
-                      if isinstance(value, collections.Mapping))
-        return super(QuerySet, self).annotate(*args, **kwargs)
+        for field, value in kwargs.items():
+            if Case.isa(value):
+                kwargs[field] = Case.defaultdict(value)
+        return super().annotate(*args, **kwargs)
 
-    def value_counts(self, alias='count'):
+    def value_counts(self, alias: str = 'count') -> 'QuerySet':
         """Return annotated value counts."""
-        if hasattr(self, '_groupby'):
-            warnings.warn("value_counts implicitly groups by fields or expressions", DeprecationWarning)
-            self = self[self._groupby]
         return self.items(*self._fields, **{alias: F.count()})
 
-    def sort_values(self, reverse=False):
+    def sort_values(self, reverse=False) -> 'QuerySet':
         """Return `QuerySet`_ ordered by selected values."""
         qs = self.order_by(*self._fields)
         return qs.reverse() if reverse else qs
@@ -384,81 +445,70 @@ class QuerySet(models.QuerySet, Lookup):
             return collections.namedtuple('Row', names)(**row)
         return row[names[0]] if self._flat else tuple(map(row.__getitem__, names))
 
-    def update(self, **kwargs):
+    def update(self, **kwargs) -> int:
         """Update extended to also handle mapping values, as a `Case`_ expression.
 
         :param kwargs: ``field={Q_obj: value, ...}, ...``
         """
-        kwargs.update((field, Case(value, default=F(field))) for field, value in kwargs.items()
-                      if isinstance(value, collections.Mapping))
-        return super(QuerySet, self).update(**kwargs)
+        for field, value in kwargs.items():
+            if Case.isa(value):
+                kwargs[field] = Case(value, default=F(field))
+        return super().update(**kwargs)
 
-    def modify(self, defaults=(), **kwargs):
+    def change(self, defaults: Mapping = {}, **kwargs) -> int:
         """Update and return number of rows that actually changed.
 
         For triggering on-change logic without fetching first.
 
-        ``if qs.modify(status=...):`` status actually changed
+        ``if qs.change(status=...):`` status actually changed
 
-        ``qs.modify({'last_modified': now}, status=...)`` last_modified only updated if status updated
+        ``qs.change({'last_modified': now}, status=...)`` last_modified only updated if status updated
 
         :param defaults: optional mapping which will be updated conditionally, as with ``update_or_create``.
         """
         return self.exclude(**kwargs).update(**dict(defaults, **kwargs))
 
-    def upsert(self, defaults={}, **kwargs):
-        """Update or insert returning number of rows or created object; faster and safer than ``update_or_create``.
+    def changed(self, **kwargs) -> dict:
+        """Return first mapping of fields and values which differ in the db.
 
-        Supports combined expression updates by assuming the identity element on insert:  ``F(...) + 1``.
-
-        :param defaults: optional mapping which will be updated, as with ``update_or_create``.
+        Also efficient enough to be used in boolean contexts, instead of ``exists``.
         """
-        lookup, params = self._extract_model_params(defaults, **kwargs)
-        params.update((field, value.rhs.value) for field, value in params.items()
-                      if isinstance(value, models.expressions.CombinedExpression))
-        update = getattr(self.filter(**lookup), 'update' if defaults else 'count')
-        try:
-            with transaction.atomic():
-                return update(**defaults) or self.create(**params)
-        except IntegrityError:
-            return update(**defaults)
+        row = self.exclude(**kwargs).values(*kwargs).first() or {}
+        return {field: value for field, value in row.items() if value != kwargs[field]}
 
-    def exists(self, count=1):
+    def exists(self, count: int = 1) -> bool:
         """Return whether there are at least the specified number of rows."""
         if count == 1:
-            return super(QuerySet, self).exists()
-        return len(self['pk'][:count] if self._result_cache is None else self) >= count
+            return super().exists()
+        return (self[:count].count() if self._result_cache is None else len(self)) >= count
 
 
 @models.Field.register_lookup
 class NotEqual(models.Lookup):
     """Missing != operator."""
+
     lookup_name = 'ne'
 
     def as_sql(self, *args):
         lhs, lhs_params = self.process_lhs(*args)
         rhs, rhs_params = self.process_rhs(*args)
-        return '{} <> {}'.format(lhs, rhs), lhs_params + rhs_params
+        return f'{lhs} <> {rhs}', (lhs_params + rhs_params)
 
 
 class Query(models.sql.Query):
     """Allow __ne=None lookup."""
-    def prepare_lookup_value(self, value, lookups, *args):
-        if value is None and lookups[-1:] == ['ne']:
-            value, lookups[-1] = False, 'isnull'
-        return super(Query, self).prepare_lookup_value(value, lookups, *args)
 
     def build_lookup(self, lookups, lhs, rhs):
         if rhs is None and lookups[-1:] == ['ne']:
             rhs, lookups[-1] = False, 'isnull'
-        return super(Query, self).build_lookup(lookups, lhs, rhs)
+        return super().build_lookup(lookups, lhs, rhs)
 
 
 class Manager(models.Manager):
     def get_queryset(self):
         return QuerySet(self.model, Query(self.model), self._db, self._hints)
 
-    def __getitem__(self, pk):
+    def __getitem__(self, pk) -> QuerySet:
         """Return `QuerySet`_ which matches primary key.
 
         To encourage direct db access, instead of always using get and save.
@@ -473,49 +523,62 @@ class Manager(models.Manager):
         """Return whether primary key is present using ``exists``."""
         return self[pk].exists()
 
-    def changed(self, pk, **kwargs):
-        """Return mapping of fields and values which differ in the db.
+    def upsert(self, defaults: Mapping = {}, **kwargs) -> Union[int, models.Model]:
+        """Update or insert returning number of rows or created object.
 
-        Also efficient enough to be used in boolean contexts, instead of ``exists``.
+        Faster and safer than ``update_or_create``.
+        Supports combined expression updates by assuming the identity element on insert:  ``F(...) + 1``.
+
+        :param defaults: optional mapping which will be updated, as with ``update_or_create``.
         """
-        row = self[pk].exclude(**kwargs).values(*kwargs).first() or {}
-        return {field: value for field, value in row.items() if value != kwargs[field]}
+        update = getattr(self.filter(**kwargs), 'update' if defaults else 'count')
+        for field, value in defaults.items():
+            expr = isinstance(value, models.expressions.CombinedExpression)
+            kwargs[field] = value.rhs.value if expr else value
+        try:
+            with transaction.atomic():
+                return update(**defaults) or self.create(**kwargs)
+        except IntegrityError:
+            return update(**defaults)
 
-    def bulk_changed(self, field, data):
+    def bulk_changed(self, field, data: Mapping, key: str = 'pk') -> dict:
         """Return mapping of values which differ in the db.
 
+        :param field: value column
         :param data: ``{pk: value, ...}``
+        :param key: unique key column
         """
-        rows = self.filter(pk__in=data)['pk', field].iterator()
+        rows = self.filter(F(key).isin(data))[key, field].iterator()
         return {pk: value for pk, value in rows if value != data[pk]}
 
-    def bulk_update(self, field, data, changed=False, conditional=False, **kwargs):
-        """Update with a minimal number of queries, by inverting the data to use ``pk__in``.
+    def bulk_change(self, field, data: Mapping, key: str = 'pk', conditional=False, **kwargs) -> int:
+        """Update changed rows with a minimal number of queries, by inverting the data to use ``pk__in``.
 
+        :param field: value column
         :param data: ``{pk: value, ...}``
-        :param changed: execute select query first to update only rows which differ;
-            more efficient if the expected percentage of changed rows is relatively small
-        :param conditional: execute a single query with a conditional expression;
-            may be more efficient if the number of rows is large (but bounded)
+        :param key: unique key column
+        :param conditional: execute select query and single conditional update;
+            may be more efficient if the percentage of changed rows is relatively small
         :param kwargs: additional fields to be updated
         """
-        if changed:
-            data = {pk: data[pk] for pk in self.bulk_changed(field, data)}
-        updates = collections.defaultdict(list)
+        if conditional:
+            data = {pk: data[pk] for pk in self.bulk_changed(field, data, key)}
+        updates = collections.defaultdict(list)  # type: dict
         for pk in data:
             updates[data[pk]].append(pk)
         if conditional:
-            kwargs[field] = {F.pk__in == tuple(updates[value]): value for value in updates}
-            return self.filter(pk__in=data).update(**kwargs)
+            kwargs[field] = {F(key).isin(tuple(updates[value])): value for value in updates}
+            return self.filter(F(key).isin(data)).update(**kwargs)
         count = 0
         for value in updates:
             kwargs[field] = value
-            count += self.filter(pk__in=updates[value]).update(**kwargs)
+            count += self.filter((F(field) != value) & F(key).isin(updates[value])).update(**kwargs)
         return count
 
 
 class classproperty(property):
     """A property bound to a class."""
+
     def __get__(self, instance, owner):
         return self.fget(owner)
 
@@ -525,8 +588,9 @@ def Value(value):
 
 
 def extract(field):
-    return field.name if isinstance(field, models.F) else \
-        Case.defaultdict(field) if isinstance(field, collections.Mapping) else field
+    if isinstance(field, models.F):
+        return field.name
+    return Case.defaultdict(field) if Case.isa(field) else field
 
 
 class Case(models.Case):
@@ -536,22 +600,32 @@ class Case(models.Case):
     :param default: optional default value or ``F`` object
     :param output_field: optional field defaults to registered ``types``
     """
-    types = {str: models.CharField, int: models.IntegerField, float: models.FloatField, bool: models.BooleanField}
+
+    types = {
+        str: models.CharField,
+        int: models.IntegerField,
+        float: models.FloatField,
+        bool: models.BooleanField,
+    }
 
     def __init__(self, conds, default=None, **extra):
         cases = (models.When(cond, Value(conds[cond])) for cond in conds)
         types = set(map(type, conds.values()))
         if len(types) == 1 and types.issubset(self.types):
             extra.setdefault('output_field', self.types.get(*types)())
-        super(Case, self).__init__(*cases, default=Value(default), **extra)
+        super().__init__(*cases, default=Value(default), **extra)
 
     @classmethod
     def defaultdict(cls, conds):
         conds = dict(conds)
         return cls(conds, default=conds.pop('default', None))
 
+    @classmethod
+    def isa(cls, value):
+        return isinstance(value, Mapping) and any(isinstance(key, models.Q) for key in value)
 
-def EnumField(enum, display=None, **options):
+
+def EnumField(enum, display: Callable = None, **options) -> models.Field:
     """Return a ``CharField`` or ``IntegerField`` with choices from given enum.
 
     By default, enum names and values are used as db values and display labels respectively,
