@@ -1,3 +1,4 @@
+import math
 import django
 from django.db import models
 from django.db.models import functions
@@ -42,32 +43,30 @@ def test_queryset(books):
     assert list(quant.sort_values(reverse=True)) == [10, 10, 2, 2, 1]
 
     now = timezone.now()
-    assert books.filter(author='B').modify({'last_modified': now}, quantity=2) == 1
-    assert len(books['last_modified'] == now) == 1
-    assert books.filter(author='B').modify({'last_modified': timezone.now()}, quantity=2) == 0
+    assert books.filter(author='B').change({'last_modified': now}, quantity=2) == 1
     assert len(books['last_modified'] == now) == 1
     books['quantity'] = {F.author == 'B': 3}
     assert set(books['quantity']) == {3, 10}
-    assert books.upsert({'quantity': 0}, pk=1) == 1
-    assert books.upsert(pk=0) == 0  # simulates race condition
-    book = books.upsert({'quantity': F.quantity + 1}, pk=0)
+    assert Book.objects.upsert({'quantity': 0}, pk=1) == 1
+    assert Book.objects.upsert(pk=0) == 0  # simulates race condition
+    book = Book.objects.upsert({'quantity': F.quantity + 1}, pk=0)
     assert book.pk == 0 and book.quantity == 1
-    assert books.upsert({'quantity': F.quantity + 1}, pk=0) == 1
-    assert books['quantity'].get(pk=0) == 2
+    with pytest.raises(TypeError):
+        books['quantity'] = {}
 
 
 def test_manager(books):
     assert 1 in Book.objects
     assert Book.objects[1]['id'].first() == 1
     assert Book.objects.bulk_changed('quantity', {3: 2, 4: 2, 5: 2}) == {4: 1}
+    assert Book.objects.bulk_changed('quantity', {'A': 5}, key='author') == {'A': 10}
     now = timezone.now()
-    assert Book.objects.bulk_update('quantity', {3: 2, 4: 2}, changed=True, last_modified=now) == 1
+    assert Book.objects.bulk_change('quantity', {3: 2, 4: 2}, last_modified=now) == 1
     timestamps = dict(books.filter(quantity=2)['id', 'last_modified'])
     assert len(timestamps) == 3 and timestamps[3] < timestamps[5] < timestamps[4] == now
-    assert Book.objects.bulk_update('quantity', {3: 2, 4: 3}, conditional=True) == 2
+    assert Book.objects.bulk_change('quantity', {3: 2, 4: 3}, key='id', conditional=True) == 1
     assert set(books.filter(quantity=2)['id']) == {3, 5}
-    assert Book.objects.changed(1, quantity=5) == {'quantity': 10}
-    assert Book.objects.changed(1, quantity=10) == {}
+    assert Book.objects[1].changed(quantity=5) == {'quantity': 10}
     del Book.objects[1]
     assert 1 not in Book.objects
 
@@ -102,8 +101,6 @@ def test_aggregation(books):
     amounts = books[{F.quantity <= 1: 'low', F.quantity >= 10: 'high'}]
     assert dict(amounts.value_counts()) == {'low': 1, None: 2, 'high': 2}
     groups = books.groupby(amount={F.quantity <= 1: 'low', F.quantity >= 10: 'high', 'default': 'medium'})
-    with pytest.warns(DeprecationWarning):
-        assert dict(groups.value_counts()) == {'low': 1, 'medium': 2, 'high': 2}
     with pytest.raises(Exception):
         Case({models.Q(): None}).output_field
 
@@ -129,6 +126,7 @@ def test_functions(books):
     book['quantity'] **= 2
     assert book['quantity'].first() == 9
 
+    assert (F.author != None) == models.Q(author__isnull=False)  # noqa: E711
     assert isinstance(F.coalesce('author', 'title'), functions.Coalesce)
     assert isinstance(F.author.concat('title'), functions.Concat)
     assert isinstance(F.author.length(), functions.Length)
@@ -139,7 +137,11 @@ def test_functions(books):
         F.title[:-10]
     with pytest.raises(AttributeError):
         F.name
-    (field, values), = transform('op', F.author.coalesce('title'), None).children
+    with pytest.raises(TypeError):
+        iter(F.title)
+    assert hash(F.title)
+    assert not (F.author == models.F('title'))
+    ((field, values),) = transform('op', F.author.coalesce('title'), None).children
     assert field == 'author__op' and values == (F.title, None)
 
     assert isinstance(F.title.greatest('author'), functions.Greatest)
@@ -156,13 +158,12 @@ def test_functions(books):
     assert books[F.author.upper == 'A']
 
 
-@pytest.mark.skipif(django.VERSION < (2,), reason='requires django >=2')
 def test_2(books):
     row = books['id', 'author'].first()
     assert (row.id, row.author) == row
-    row = books['author', ].min()
+    row = books['author',].min()
     assert (row.author__min,) == row
-    key, values = next(iter(books['quantity', ].groupby('author')))
+    key, values = next(iter(books['quantity',].groupby('author')))
     assert next(values).quantity
     assert dict(books[F.author.find('A')].value_counts()) == {-1: 3, 0: 2}
 
@@ -178,8 +179,12 @@ def test_2(books):
     assert isinstance(F.quantity.rank(), functions.Rank)
     assert isinstance(F.quantity.row_number(), functions.RowNumber)
 
+    point = 'POINT(0 0)'
+    if gis:
+        assert isinstance(F.location.azimuth(point), gis.functions.Azimuth)
+        assert isinstance(F.location.line_locate_point(point), gis.functions.LineLocatePoint)
+
 
-@pytest.mark.skipif(django.VERSION < (2, 1), reason='requires django >=2.1')
 def test_2_1():
     assert (F.quantity.chr == '').children == [('quantity__chr', '')]
     assert isinstance(F.quantity.chr(), functions.Chr)
@@ -196,6 +201,64 @@ def test_2_1():
     assert isinstance(F.author.ljust(1), functions.LPad)
     assert isinstance(F.author.rjust(1), functions.RPad)
 
+    if gis:
+        assert isinstance(F.location.force_polygon_cw(), gis.functions.ForcePolygonCW)
+
+
+def test_2_2():
+    assert isinstance(F.x.nullif('y'), functions.NullIf)
+    assert isinstance(reversed(F.x), functions.Reverse)
+    assert isinstance(abs(F.x), functions.Abs)
+    assert isinstance(F.x.acos(), functions.ACos)
+    assert isinstance(F.x.asin(), functions.ASin)
+    assert isinstance(F.x.atan(), functions.ATan)
+    assert isinstance(F.x.atan2('y'), functions.ATan2)
+    assert isinstance(math.ceil(F.x), functions.Ceil)
+    assert isinstance(F.x.cos(), functions.Cos)
+    assert isinstance(F.x.cot(), functions.Cot)
+    assert isinstance(F.x.degrees(), functions.Degrees)
+    assert isinstance(F.x.exp(), functions.Exp)
+    assert isinstance(math.floor(F.x), functions.Floor)
+    assert isinstance(F.x.log(), functions.Log)
+    assert isinstance(F.x.log(2), functions.Log)
+    assert isinstance(F.x % 2, functions.Mod)
+    assert isinstance(2 % F.x, functions.Mod)
+    assert isinstance(F.pi, functions.Pi)
+    assert isinstance(F.x ** 2, functions.Power)
+    assert isinstance(2 ** F.x, functions.Power)
+    assert isinstance(F.x.radians(), functions.Radians)
+    assert isinstance(round(F.x), functions.Round)
+    assert isinstance(F.x.sin(), functions.Sin)
+    assert isinstance(F.x.sqrt(), functions.Sqrt)
+    assert isinstance(F.x.tan(), functions.Tan)
+
+    assert isinstance(F.x.acos, F)
+    assert isinstance(F.x.asin, F)
+    assert isinstance(F.x.atan, F)
+    assert isinstance(F.x.atan2, F)
+    assert isinstance(F.x.cos, F)
+    assert isinstance(F.x.cot, F)
+    assert isinstance(F.x.degrees, F)
+    assert isinstance(F.x.exp, F)
+    assert isinstance(F.x.radians, F)
+    assert isinstance(F.x.sin, F)
+    assert isinstance(F.x.sqrt, F)
+    assert isinstance(F.x.tan, F)
+
+
+@pytest.mark.skipif(django.VERSION < (3,), reason='requires django >=3')
+def test_3():
+    assert isinstance(F.x.sign(), functions.Sign)
+    assert isinstance(F.x.md5(), functions.MD5)
+    assert isinstance(F.x.sha1(), functions.SHA1)
+    assert isinstance(F.x.sha224(), functions.SHA224)
+    assert isinstance(F.x.sha256(), functions.SHA256)
+    assert isinstance(F.x.sha384(), functions.SHA384)
+    assert isinstance(F.x.sha512(), functions.SHA512)
+
+    assert isinstance(F.x.sign, F)
+    assert isinstance(F.x.md5, F)
+
 
 def test_lookups(books):
     assert books[F.last_modified.year == timezone.now().year].count() == 5
@@ -203,7 +266,7 @@ def test_lookups(books):
     assert isinstance(F.quantity.max(), models.Max)
     assert isinstance(F.quantity.sum(), models.Sum)
     assert isinstance(F.quantity.mean(), models.Avg)
-    assert str(F.quantity.count()) == "Count(F(quantity), distinct=False)"
+    assert str(F.quantity.count()).startswith('Count(F(quantity)')
     assert str(F.count(distinct=True)) == "Count('*', distinct=True)"
     assert isinstance(F.quantity.var(sample=True), models.Variance)
     assert isinstance(F.quantity.std(sample=True), models.StdDev)
@@ -212,7 +275,7 @@ def test_lookups(books):
     assert str(F.all(exprs)) == "(AND: ('author__contains', 'A'), ('author__contains', 'B'))"
 
     authors = books['author']
-    assert set(authors.in_('A', 'B')) == {'A', 'B'}
+    assert set(authors.isin('AB')) == {'A', 'B'}
     assert set(authors.iexact('a')) == {'A'}
     assert set(authors.contains('A')) == {'A'}
     assert set(authors.icontains('a')) == {'A'}
@@ -270,6 +333,7 @@ def test_spatial_lookups():
 @pytest.mark.skipif(not gis, reason='requires spatial lib')
 def test_spatial_functions(books):
     from django.contrib.gis.geos import Point
+
     point = Point(0, 0, srid=4326)
 
     assert isinstance(F.location.area, gis.functions.Area)
@@ -280,7 +344,6 @@ def test_spatial_functions(books):
     assert isinstance(F.location.bounding_circle(), gis.functions.BoundingCircle)
     assert isinstance(F.location.centroid, gis.functions.Centroid)
     assert isinstance(F.location.envelope, gis.functions.Envelope)
-    assert isinstance(F.location.force_rhr(), gis.functions.ForceRHR)
     assert isinstance(F.location.geohash(), gis.functions.GeoHash)
     assert isinstance(F.location.make_valid(), gis.functions.MakeValid)
     assert isinstance(F.location.mem_size, gis.functions.MemSize)
@@ -297,10 +360,15 @@ def test_spatial_functions(books):
     dist = F.location.distance(point)
     assert isinstance(dist, gis.functions.Distance)
     fields, items = zip(*F.all([(dist < 0), (dist <= 0), (dist > 0), (dist >= 0), dist.within(0)]).children)
-    assert fields == ('location__distance_lt', 'location__distance_lte',
-                      'location__distance_gt', 'location__distance_gte', 'location__dwithin')
+    assert fields == (
+        'location__distance_lt',
+        'location__distance_lte',
+        'location__distance_gt',
+        'location__distance_gte',
+        'location__dwithin',
+    )
     assert items == ((point, 0),) * 5
-    (field, values), = (F.location.distance(point) > 0).children
+    ((field, values),) = (F.location.distance(point) > 0).children
     assert values == (point, 0)
 
     assert isinstance(F.location.difference(point), gis.functions.Difference)
@@ -324,12 +392,14 @@ def test_enum():
     class gender(enum.Enum):
         M = 'Male'
         F = 'Female'
+
     assert gender.max_length == 1
     assert dict(gender.choices) == {'M': 'Male', 'F': 'Female'}
 
     class Gender(enum.Enum):
         MALE = 0
         FEMALE = 1
+
     gender = EnumField(Gender, str.title)
     assert isinstance(gender, models.IntegerField)
     assert dict(gender.choices) == {0: 'Male', 1: 'Female'}
